import { EventEmitter } from '../../stencil-public-runtime';
import { CaptureVisionRouter } from 'dynamsoft-capture-vision-router';
export interface DetectedQuadResult {
    location: Quad;
    confidenceAsDocumentBoundary: number;
}
export interface Quad {
    points: [Point, Point, Point, Point];
}
export interface Point {
    x: number;
    y: number;
}
export interface Rect {
    x: number;
    y: number;
    width: number;
    height: number;
}
export interface CropOptions {
    perspectiveTransform?: boolean;
    colorMode?: "binary" | "gray" | "color";
    selection?: Quad | Rect;
    source?: Blob | string | HTMLImageElement | HTMLCanvasElement;
}
interface TouchDataStore {
    scale?: number;
    originalScale?: number;
    point1?: Point;
    point2?: Point;
    moveable?: boolean;
}
export declare class ImageCropper {
    handlers: number[];
    polygonMouseDown: boolean;
    polygonMouseDownPoint: Point;
    touchDataStore: TouchDataStore | undefined;
    svgMouseDownPoint: Point | undefined;
    handlerMouseDownPoint: Point;
    root: HTMLElement;
    containerElement: HTMLElement;
    svgElement: SVGElement;
    imgElement: SVGImageElement;
    canvasElement: HTMLCanvasElement;
    originalPoints: [Point, Point, Point, Point];
    cvr: CaptureVisionRouter | undefined;
    usingTouchEvent: boolean;
    usingQuad: boolean;
    previousTouchedTime: number;
    previousTouchedPoint: {
        x: number;
        y: number;
    };
    scaledAfterDoubleTap: boolean;
    img?: HTMLImageElement;
    rect?: Rect;
    quad?: Quad;
    license?: string;
    hidefooter?: string;
    handlersize?: string;
    inactiveSelections?: (Quad | Rect)[];
    draggingmode?: "x-only" | "y-only";
    viewBox: string;
    activeStroke: number;
    inActiveStroke: number;
    selectedHandlerIndex: number;
    points: [Point, Point, Point, Point];
    offsetX: number;
    offsetY: number;
    scale: number;
    confirmed?: EventEmitter<void>;
    canceled?: EventEmitter<void>;
    selectionClicked?: EventEmitter<number>;
    imageLoaded?: EventEmitter<void>;
    componentDidLoad(): void;
    watchImgPropHandler(newValue: HTMLImageElement): void;
    watchRectPropHandler(newValue: Rect): void;
    getPointsFromRect(rect: Rect): [Point, Point, Point, Point];
    watchQuadPropHandler(newValue: Quad): void;
    onCanceled(): void;
    onConfirmed(): void;
    getPointsData(): string;
    renderFooter(): any;
    rendenInactiveSelections(): any;
    onSelectionClicked(index: number): void;
    getPointsDataFromSelection(selection: Quad | Rect): string;
    renderHandlers(): any;
    getHandlerPos(index: number, key: string): number;
    getHandlerSize(): number;
    onSVGTouchStart(e: TouchEvent): void;
    onSVGTouchEnd(): void;
    onSVGTouchMove(e: TouchEvent): void;
    pinchAndZoom(e: TouchEvent): void;
    onContainerMouseUp(): void;
    onSVGMouseDown(e: MouseEvent): void;
    onContainerWheel(e: WheelEvent): void;
    onContainerTouchStart(e: TouchEvent): void;
    initTouchDataStore(e: TouchEvent): void;
    onContainerTouchEnd(): void;
    onContainerTouchMove(e: TouchEvent): void;
    getPanAndZoomStyle(): string;
    onSVGMouseMove(e: MouseEvent): void;
    panSVG(e: TouchEvent | MouseEvent): void;
    handleMoveEvent(e: MouseEvent | TouchEvent): void;
    restrainPointsInBounds(points: [Point, Point, Point, Point], imgWidth: number, imgHeight: number): void;
    checkDoubleTap(e: TouchEvent): void;
    onPolygonMouseDown(e: MouseEvent): void;
    onPolygonMouseUp(e: MouseEvent): void;
    onPolygonTouchStart(e: TouchEvent): void;
    onPolygonTouchEnd(e: TouchEvent): void;
    onHandlerMouseDown(e: MouseEvent, index: number): void;
    onHandlerMouseUp(e: MouseEvent): void;
    onHandlerTouchStart(e: TouchEvent, index: number): void;
    onHandlerPointerDown(e: PointerEvent, index: number): void;
    getPointIndexFromHandlerIndex(index: number): 1 | 2 | 3 | 0 | -1;
    getMousePosition(event: any, svg: any): {
        x: number;
        y: number;
    };
    isSafari(): boolean;
    getRatio(): number;
    resetStates(): Promise<void>;
    getAllSelections(convertTo?: "rect" | "quad"): Promise<(Quad | Rect)[]>;
    getPoints(): Promise<[Point, Point, Point, Point]>;
    getQuad(): Promise<Quad>;
    getRect(): Promise<Rect>;
    getRectFromPoints(points: Point[]): Rect;
    getCroppedImage(options: CropOptions): Promise<string>;
    getImageFromBlob(source: Blob): Promise<HTMLImageElement>;
    getImageFromDataURL(source: string): Promise<HTMLImageElement>;
    detect(source: string | HTMLImageElement | Blob | HTMLCanvasElement): Promise<DetectedQuadResult[]>;
    fitWidth(): Promise<void>;
    fitWindow(): Promise<void>;
    fitActualSize(): Promise<void>;
    goToTop(): Promise<void>;
    initCVR(): Promise<void>;
    getSVGWidth(): number | "100%";
    onSVGPointerMove(e: PointerEvent): void;
    onSVGPointerDown(e: PointerEvent): void;
    onSVGPointerUp(e: PointerEvent): void;
    onSVGPointerLeave(): void;
    onPolygonPointerDown(e: PointerEvent): void;
    onPolygonPointerUp(e: PointerEvent): void;
    onImageLoaded(): void;
    render(): any;
}
export {};
